from .abc import Message, patcher
from .text import ABCTextMessageRule
import typing
import vbml

PatternLike = typing.Union[str, vbml.Pattern]


class Markup(ABCTextMessageRule):
    def __init__(self, patterns: typing.Union[PatternLike, typing.List[PatternLike]]):
        if not isinstance(patterns, list):
            patterns = [patterns]
        self.patterns = [
            vbml.Pattern(pattern) if isinstance(pattern, str) else pattern
            for pattern in patterns
        ]

    async def check(self, message: Message, ctx: dict) -> bool:
        for pattern in self.patterns:
            response = patcher.check(pattern, message.text)
            if response in (False, None):
                continue
            elif isinstance(response, dict):
                ctx.update(response)
            return True
        return False



from .abc import ABCRule
from telegrinder.modules import json
from telegrinder.types import Update


class CallbackDataEq(ABCRule):
    def __init__(self, value: str):
        self.value = value

    async def check(self, event: Update, ctx: dict) -> bool:
        return event.callback_query.data == self.value


class CallbackDataJsonEq(ABCRule):
    def __init__(self, d: dict):
        self.d = d

    async def check(self, event: Update, ctx: dict) -> bool:
        if not event.callback_query.data:
            return False
        try:
            # todo: use msgspec
            return json.loads(event.callback_query.data) == self.d
        except:
            return False
